from Acquisition import aq_inner
from OFS.SimpleItem import SimpleItem
from zope.component import adapts, getUtility
from zope.component.interfaces import ComponentLookupError
from zope.interface import Interface, implements
from zope.formlib import form
from zope import schema

from plone.app.contentrules.browser.formhelper import AddForm, EditForm 
from plone.app.vocabularies.groups import GroupsSource
from plone.app.vocabularies.users import UsersSource
from plone.app.form.widgets.uberselectionwidget import UberMultiSelectionWidget
from plone.contentrules.rule.interfaces import IRuleElementData, IExecutable

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import safe_unicode

from Products.PDFtoOCR.interfaces import IIndex

class IOCRAction(Interface):
    """Definition of the configuration available for the OCR action
    """

class OCRAction(SimpleItem):
    """
    The implementation of the action defined before
    """
    implements(IOCRAction, IRuleElementData)

    element = 'plone.actions.PDFtoOCR'

    @property
    def summary(self):
        return _(u"DO OCR STUFF")

    

class OCRActionExecutor(object):
    """The executor for this action.
    """
    implements(IExecutable)
    adapts(Interface, IOCRAction, Interface)

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        context = aq_inner(self.context)

        obj = self.event.object

        contentType = obj.getContentType()

        util = getUtility(IIndex, 'pdf_to_ocr_indexer')
        util.addFile(obj.UID())
        
        return True


class OCRAddForm(AddForm):
    """
    An add form for the OCR action
    """
    form_fields = form.FormFields(IOCRAction)
    label = _(u"Add OCR Action for indexing PDF documents")
    description = _(u"Index PDF documents using OCR. Used for document where pdf2text doesn't work. Don't forget to setup cron4plone")
    form_name = _(u"Add PDF2OCR action")

    def create(self, data):
        a = OCRAction()
        form.applyChanges(a, self.form_fields, data)
        return a



class OCREditForm(EditForm):
    """
    An edit form for the OCR action
    """
    form_fields = form.FormFields(IOCRAction)
    label = _(u"Add OCR Action for indexing PDF documents")
    description = _(u"Index PDF documents using OCR. Used for document where pdf2text doesn't work. Don't forget to setup cron4plone")
    form_name = _(u"Add PDF2OCR action")






import logging

from zope.app import zapi
from zope.component import getUtility 
from zope.interface import implements
from persistent import Persistent
from persistent.list import PersistentList

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import ISiteRoot
from OFS.SimpleItem import SimpleItem

from Products.PDFtoOCR.interfaces import IIndex

logger = logging.getLogger('Products.PDFtoOCR')


class Index(Persistent, SimpleItem):
    """
    """ 

    def __init__(self):
        self._fileList = set()
        self._history = PersistentList()

    def reinit(self):
        self.__init__()

    def _changes(self):
        self._p_changed = 1 # for persistence..    
 
    def addFile(self, uid):
        self._fileList.add(uid)
        self._changes()

    def updateHistory(self, uid):
        self._history.append(uid)
        if len(self._history) > 10:
            self._history = self._history[:10]

    def doIndex(self):
        """ Index the PDF documents in the filelist queue
        """
        documents = []
        filesDone = set()
        fileList = self._fileList
        portal = getUtility(ISiteRoot)
        catalog = getToolByName(portal, 'portal_catalog')

        logger.info('Start OCR indexing of pdf documents')
        logger.info('Got %s docs in file list' % len(fileList))

        # Proces pdf files for indexing
        # remove all other files
        for uid in fileList: 
            results = portal.uid_catalog(UID=uid)

            if results:
                file = results[0].getObject()
                if file.getContentType() == 'application/pdf':
                   self.ocrIndexPDF(file)
            filesDone.add(uid)

        # Remove all non-pdf documents
        self._fileList = self._fileList - filesDone
        self._changes() 
        logger.info('OCR indexing of pdf documents done')


    def doReindex(self):
        """ Indexes all pdf files
        Warning: this can take a while when you have many pdf docs
        """
        logger.info('Start OCR reindexing of pdf documents')
        portal = getUtility(ISiteRoot)
        catalog = getToolByName(portal, 'portal_catalog')
        results = catalog(Type='File')
        logger.info('Got %s docs in file list' % len(results))

        for brain in results:
            document = brain.getObject()
            if document.getContentType() == 'application/pdf':
                self.ocrIndexPDF(document)
        
        logger.info('OCR reindexing of pdf documents done')

    def ocrIndexPDF(self, document):
        """ Does OCR indexing on pdf documents
        """
        logger.info('OCR processing %s' % document.getId())
    
        language = document.getDocLanguage()
        if language == 'default':
            portal = getUtility(ISiteRoot)
            properties = getToolByName(portal, 'portal_properties')
            language = properties.site_properties.default_language

        output = str(document.getFile().data)
        text = self.pdf_to_ocr(output, language)
        # Do we need this?
        #words = self._process_words(text, language)

        if text:
            document.setTextFromOcr(text)
            document.reindexObject() 
            self.updateHistory(document.UID())




from Products.ExternalMethod.ExternalMethod import ExternalMethod

from Products.PDFtoOCR.interfaces import IIndex
from Products.PDFtoOCR.index import Index

def installExternalMethods(self):
    id = 'pdf_to_ocr'
    title = 'pdf_to_ocr'
    if not id in self.objectIds():
        em = ExternalMethod(
             id=id,
             title=title,
             module='Products.PDFtoOCR.ocrfile',
             function='ocrfile')
        self._setObject(id, em)

def setupVarious(context):

    # Ordinarily, GenericSetup handlers check for the existence of XML files.
    # Here, we are not parsing an XML file, but we use this text file as a
    # flag to check that we actually meant for this import step to be run.
    # The file is found in profiles/default.

    if context.readDataFile('Products.PDFtoOCR_various.txt') is None:
        return
    print 666

    # Add additional setup code here
    portal = context.getSite()
    sm = portal.getSiteManager()

    if not sm.queryUtility(IIndex, name='pdf_to_ocr_indexer'):
        sm.registerUtility(Index(),
                           IIndex,
                           'pdf_to_ocr_indexer')

    installExternalMethods(portal)



from zope.component import getUtility 

from Products.Five import BrowserView
from Acquisition import aq_inner
from Products.PDFtoOCR.interfaces import IIndex

class DoIndexView(BrowserView):
    """Just some testing / trying out.
    """

    def index(self):
        util = getUtility(IIndex ,'pdf_to_ocr_indexer')
        util.doIndex()
        
    def reindex(self):
        util = getUtility(IIndex ,'pdf_to_ocr_indexer')
        util.doReindex()
        
    def reinit(self):
        util = getUtility(IIndex ,'pdf_to_ocr_indexer')
        util.reinit()
        return 'Queue is reset!'

    def getUIDInfo(self, uid):
        context = aq_inner(self.context)
        results = context.uid_catalog(UID=uid)
        doc = dict()
        if results:
            item = results[0]
            doc['Title'] = item.Title
            doc['Description'] = item.Description()
            doc['id'] = item.id
            doc['url'] = item.getURL()
        return doc 


    def queueStatus(self):
        util = getUtility(IIndex ,'pdf_to_ocr_indexer')
        files = util._fileList
        return  [self.getUIDInfo(f) for f in files]

    def historyStatus(self):
        util = getUtility(IIndex ,'pdf_to_ocr_indexer')
        files = util._history
        return  [self.getUIDInfo(f) for f in files]







def ocrfile(f, language=None):
    """ This method converts raw data of a pdf file to
        tiff images. Each pdf page gets converted to a tiff 
        image using ghostsscript.

        Using tesseract OCR is done on the images and finally
        the text is returned.
    """

    import urllib
    import os
    import sys
    import tempfile

    gs = '/usr/bin/gs'
    tess = '/usr/local/bin/tesseract'

    dir1 = tempfile.mkdtemp()
    pdffilename = dir1 + '/document.pdf'
    imagefilename = dir1 + '/image_%04d.tif'
    txtfilename = dir1 + '/output'
    txtoutput = ""

    file = open(pdffilename, "wb")
    file.write(f)
    file.close()
 
    # If the text is extractable with pdftotext don't
    # do ocr processing.
    # TODO: check somewhere in Plone is the pdftotext is done..
    pdftotext = os.popen2('pdftotext -enc UTF-8 %s -' % pdffilename)
    pdftext = pdftotext[1].read()
    if pdftext.strip():
        return None

    gsParams = [gs,
                '-dSAFER',
                '-dBATCH',
                '-dNOPAUSE',
                '-sDEVICE=tiffgray',
                '-r250',
                '-dTextAlphaBits=4',
                '-dGraphicsAlphaBits=4',
                '-dMaxStripSize=8192',
                '-sOutputFile=%s' % imagefilename,
                pdffilename,
                ]

    # Convert pdf to tiff images, each page is one tiff file
    os.spawnv(os.P_WAIT, gs, gsParams)
    os.remove(pdffilename)

    dirContents = os.listdir(dir1)
    dirContents.sort()

    # tesseract has other language abbrevations
    if language:
        tesseractLangs = {'de': 'deu',
                          'en': 'eng',
                          'fr': 'fra',
                          'it': 'ita',
                          'nl': 'nld',
                          'po': 'por'}
        language = tesseractLangs.get(language, 'eng')

    for image in dirContents:
        if image.startswith('image'):
            imagefilename = '%s/%s' % (dir1 , image)
             
            tessParams = [tess,
                          imagefilename,
                          txtfilename]
            if language:
                tessParams.append('-l')
                tessParams.append(language)

            # Do the OCR processing
            os.spawnv(os.P_WAIT, tess, tessParams)

            file = open(txtfilename + '.txt', "r")
            txtoutput += file.read()
            file.close()

            os.remove(txtfilename + '.txt')
            os.remove(imagefilename)

    # Clean-up temp dir
    os.rmdir(dir1)
    return txtoutput

if __name__ == '__main__':
    # stuff for testing this external method 
    file = open('/tmp/gasunie.pdf', "r")
    pdf =  file.read()
    output = ocrfile(None,pdf, 'deu')
    file.close()

    file = open('/tmp/output.txt', "wb")
    file.write(output)
    file.close()






from Products.Archetypes.atapi import *
from Products.CMFCore import utils as cmfutils

# Unleash the crazy monkeys!
import patches

def initialize(context):
    """Initializer called when used as a Zope 2 product."""




from Products.ATContentTypes.content import  schemata
from Products.ATContentTypes.content.schemata import ATContentTypeSchema

from Products.ATContentTypes.content import file
from Products.ATContentTypes.content.file import ATFile
from Products.ATContentTypes.content.file import ATFileSchema

#from Products.ATContentTypes.content.document import ATDocument
#from Products.ATContentTypes.content.document import ATDocumentSchema
from Products.Archetypes import atapi
from Products.Archetypes.atapi import BaseSchema

from Products.Archetypes.Schema import Schema
from Products.Archetypes.atapi import StringField,TextField
from Products.Archetypes.atapi import TextAreaWidget, SelectionWidget
from Products.ATContentTypes.config import PROJECTNAME
from Products.ATContentTypes.content.base import registerATCT

# Tesseract supported languages
langs = [('default','-- Site default --'),
         ('de','Deutsch'), ('en','English'), ( 'fr','Français'),
         ('it','Italiano'), ('nl','Nederlands'),('po','Português')]

extra_schema = Schema((
        TextField(
            name='textFromOcr',
            widget=TextAreaWidget(
                label="Text from OCR",
                description="Text from PDF documents using OCR",
                visible = {"edit": "invisible", "view": "invisible"},
            ),
            searchable=1,
        ),
        StringField('docLanguage',
            vocabulary = langs,
            default = 'default',
            widget=SelectionWidget(
                label="Language", 
                description="Language is used for ocr processing a pdf document",
                format='select')
            ),
))


if not ATFileSchema.has_key('textFromOcr'):
    ATFile.schema = ATFile.schema + extra_schema.copy()

registerATCT(ATFile, PROJECTNAME)

print "patched ATContentTypeSchema (ocr and language fields added to ATFile)"

